/* main.c
 * This file is part of Vara digital painting program
 * Copyright 2021, 2023, 2024 Nandakumar Edamana
 * Distributed under GNU General Public License, version 3
 */ 

#include <assert.h>
#include <stdarg.h>
#include <stdbool.h>

#include "main.h"

int main(int argc, char * *argv)
{
	gtk_init(0, NULL);
	Test testtype = TEST_NONE;
	_Bool test_vara_logo_exit = false;
	int test_vara_logo_size = 256;
	int argi = 1;
	while(argi < argc) {
		char * opt = argv[argi];
		if(0 == strcmp(opt, "--static-cursor")) {
		{
			clopts.static_cursor = true;
		}
		}
		else if(0 == strcmp(opt, "--test")) {
		{
			testtype = TEST_UI;
		}
		}
		else if(0 == strcmp(opt, "--test-icons")) {
		{
			testtype = TEST_ICONS;
		}
		}
		else if(0 == strcmp(opt, "--test-sunset")) {
		{
			testtype = TEST_SUNSET;
		}
		}
		else if(0 == strcmp(opt, "--test-vara-logo")) {
		{
			testtype = TEST_VARALOGO;
		}
		}
		else if(0 == strcmp(opt, "--test-vara-logo-exit")) {
		{
			test_vara_logo_exit = true;
		}
		}
		else if(0 == strcmp(opt, "--test-vara-logo-size")) {
		{
			if((argi + 1) >= argc) {
				fprintf(stderr, "%s: --test-vara-logo-size expects argument\n", argv[0]);
				exit(EXIT_FAILURE);
			}

			test_vara_logo_size = atoi(argv[argi + 1]);
			if(test_vara_logo_size < 1) {
				fprintf(stderr, "%s: --test-vara-logo-size expects a positive integer\n", argv[0]);
				exit(EXIT_FAILURE);
			}

			argi += 1;
		}
		}
		else if(0 == strcmp(opt, "--version")) {
		{
			printf("%s %s\n", PROGRAM_NAME, VERSION);
			exit(EXIT_SUCCESS);
		}
		}
		else {
		{
			fprintf(stderr, "%s: unknown command-line option: %s\n", argv[0], opt);
			exit(EXIT_FAILURE);
		}
		}
		argi += 1;
	}

	ImageDocument *doc = (ImageDocument *) malloc(sizeof(ImageDocument));
	if(doc == NULL) {
		perror(NULL);
		exit(EXIT_FAILURE);
	}

	image_document_construct(doc, DEFWIDTH_DOC, DEFHEIGHT_DOC);
	myapp *app = (myapp *) malloc(sizeof(myapp));
	if(app == NULL) {
		perror(NULL);
		exit(EXIT_FAILURE);
	}

	myapp_construct(app, doc);
	switch(testtype) {
	case TEST_NONE:
	{
		break;
	}
	case TEST_UI:
	{
		testmain_general(app);
		break;
	}
	case TEST_ICONS:
	{
		testmain_draw_icons(app);
		break;
	}
	case TEST_SUNSET:
	{
		testmain_draw_sunset(app);
		break;
	}
	case TEST_VARALOGO:
	{
		app->test_vara_logo_exit = test_vara_logo_exit;
		app->test_vara_logo_size = test_vara_logo_size;
		testmain_draw_vara_logo(app);
		break;
	}
	}

	gtk_main();

	return 0;
}

